
// ****************************************************************************
//                                 
//                               Configurations
//
// ****************************************************************************

// Set CPU frequency in the makefile file - variable F_CPU

// Set CPU fuses (with crystal 16.0000 MHz):
//   LOW = 0xEF
//   HIGH = 0xC9

// Or as alternative, internal RC oscilator, fuse HIGH=0xD9, fuse LOW:
//   8 Mhz = 0xE4, 4 MHz = 0xE3, 2 MHz = 0xE2, 1 MHz = 0xE1

// ----------------------------------------------------------------------------
//                                 Debug switches
// ----------------------------------------------------------------------------

//....#define DO_CALIBREF	// do voltage reference calibration using external reference TS431 2.5V, connected to GND and Rx input
			// Wait a minute to stabilise values, then do set ADC_REF_U, ADC_BANDGAP_U, ADC_VCC_U in config.h
			// You can adjust voltage reference value ADC_CAL_U in config.h (2.5000V default).

//#define DO_ADCSPEED 7	// do ADC speed test - select speed 0..7; displays delay of selected ADC speed,
			//  time of ADC_Sample and SetRes (set SETRES_US constant)

//#define DO_CLOCKTEST	// do clock test - compare timer with seconds of real time clocks,
			//   displays delay of GetTime (set GETTIME_US constant), waitus and waitms

//#define DO_PRECTEST 10 // do ADC speed precission test (displays value*10 + errors in 0.1 samples of divider 6, 5, 4, 3, 2, 1)
			// select test resistors: 1=R1+R2, 2=R2+R3, 3=R3+R4, 4=R4+R5, 5=R5+R6,
			//      6=R6+R7, 7=R7+R8, 8=R7+R6, 9=R5+R4, 10=R3+R2

//...#define DO_SELTEST	// do ADC selection test (displays value after reference selection, depending on delay)
			//  On first row should be values approx. 492, on second row values approx. 255.
			//  Values on rows should be approx. the same as the first ones on the rows.

//#define DO_RPU	// check pull-up resistor

//#define DO_CHECKLEN	// get length of checks in ms: RC_Check, RC_DetCap, RC_CheckRes, RC_CheckESR

//#define DO_CHECKRES 9 // check one specific reference resistor to measure resistors (set 1..9 to check R1..R8, R8B)
//#define DO_CHECKRESFAST // flag - do DO_CHECKRES in fast mode 
			// If used without DO_CHECKRES flag - use fast auto mode

//#define DO_CHECKCAP 1	// check one specific reference resistor to measure capacitors (set 1..8 to check R1..R8)
//#define DO_CHECKCAPFAST // flag - do DO_CHECKCAP in fast mode 
			// If used without DO_CHECKCAP flag - use fast auto mode

//#define DO_CHECKFREQ	// display LC frequency instead of U voltage

// ----------------------------------------------------------------------------
//                                ADC voltages
// ----------------------------------------------------------------------------

// voltage of internal reference in 0.1 mV, used for ADC sampling (2.56V)
//#define ADC_REF_U	27192 // datasheet 25600

// voltage of internal reference in 0.1 mV, measured on channel 14 (1.23V)
//#define ADC_BANDGAP_U	13010 // datasheet 12300

// default Vcc voltage in 0.1 mV (5.0 V)
//#define ADC_VCC_U	50743 // datasheet 50000

// voltage multiplier * VOLT_DIV
//#define VOLT_MUL 	7998UL // default 

// voltage of external calibration in 0.1 mV (2.5V)
//#define ADC_CAL_U	25000 // datasheet 25000

// ----------------------------------------------------------------------------
//                                RE Resistors
// ----------------------------------------------------------------------------

// MCU internal resistor value (24 Ohm)
#define R0_VAL 240UL // internal resistor, default (x 10)

// test resistor values (in ohms)
#define R1_VAL 1000UL // 100 x 10
#define R2_VAL 4700UL // 470 x 10
#define R3_VAL 2200UL // 2K2
#define R4_VAL 10000UL // 10K
#define R5_VAL 47000UL // 47K
#define R6_VAL 220000UL // 220K
#define R7_VAL 1000000UL // 1M
#define R8_VAL 4700000UL // 4M7
#define R8B_VAL 4710000UL // 4M7 at 22M

// default resistor tare value (in 0.1 Ohm)
#define R_TARE 0L

// default capacitor tare value (in 0.1 pF)
#define C_TARE	600L

// ----------------------------------------------------------------------------
//                                 LC Oscilator
// ----------------------------------------------------------------------------

// Calibration:
// - if "big L" is high or "small C" is high, increase C0REF
// - if "big C" is high or "small L" is high, increase L0REF

// default reference C in 0.1pF (scale = -13)
#define C0REF 10000UL // 1nF

// default reference L in 10nH (scale = -8)
#define L0REF 6800UL // 68uH

// default inductor tare value (in 10nH)
#define L_TARE 0L

// default U tare value (in mV)
#define U_TARE 0L
