
// ****************************************************************************
//                                 
//                                  ADC
//
// ****************************************************************************

#include "include.h"

// current selected channel (255=none)
u8 ADC_ChanSel = 255;

// current selected speed (255=none)
u8 ADC_SpeedSel = 255;

// read ADC sample, range 0..1023
u16 ADC_Sample()
{
	ADCSRA |= (1 << ADSC); // start conversion
	while (ADCSRA & (1 << ADSC)) {}; // wait until conversion is done
	return (ADCW & 0x3ff); // read ADC result
}

// select new ADC channel (dummy conversion and wait if channel has been changed)
//  channel = ADC input channel number
void ADC_Sel(u8 channel)
{
	// select input channel and reference voltage
	ADMUX = channel | ADC_REF_VCC;

	// dummy conversion if channel has been changed (needed with speed 4 and faster)
	if (channel != ADC_ChanSel)
	{
		// read dummy sample
		ADC_Sample();

		// save new current channel
		ADC_ChanSel = channel;
	}
}

// Read ADC samples, scale 0..1023*samples
// (should take 20 ms to eliminate industrial noise 50 Hz)
// read speed - see ADCSpeedTest function
u32 ADC_Read(u16 samples)
{
	u32 val = 0;
	for (; samples > 0; samples--) val += ADC_Sample();
	return val;
}

// set ADC conversion speed (0..7)
// - speed 0 or 1 on 16 MHz does not work
void ADC_Speed(u8 speed)
{
	ADCSRA = (1<<ADEN) | speed;

	// dummy conversion if speed has been changed
	if (speed != ADC_SpeedSel)
	{
		// read dummy sample
		ADC_Sample();

		// set new speed
		ADC_SpeedSel = speed;
	}
}

// ADC initialize
void ADC_Init()
{
	// set ADC pins as input, no pull-up resistors
	ADC_DDR &= ~ADC_PINS;
	ADC_PORT &= ~ADC_PINS;
}
