
// ****************************************************************************
//                                 
//                                  ADC
//
// ****************************************************************************

#define ADC_PORT	PORTC	// port data register
#define ADC_DDR		DDRC	// port data direction register
#define ADC_PIN		PINC	// port input pins register

#define ADC_PINS	(B4+B6) // ADC input pins

// ADC reference selection
#define ADC_REF_VCC	(1 << REFS0)	// Vcc (5.0V)
//#define ADC_REF_REF	((1 << REFS1) | (1 << REFS0)) // internal reference (2.56V)

// ADC reference selection bit mask
#define ADC_REF_MASK	((1 << REFS1) | (1 << REFS0))

// ADC MUX input channels
#define ADC_MUX_IN	4	// IN input channel (measure resistors)
#define ADC_MUX_U	6	// voltage input channel
#define ADC_MUX_BANDGAP	14	// reference 1.23V input channel
#define ADC_MUX_GND	15	// GND input channel

// ADC divider for precise reading (0..7)
#define ADC_SPEED	7	// divider 128

// number of ADC samples for precise measurement with ADC_SPEED (max. 350 to avoid overflow), takes 20 ms
// - time of measure should be 1 period of 50 Hz
#if F_CPU >= 14000000
#define ADC_SAMPLES 179 // 16 MHz
#elif F_CPU >= 10000000
#define ADC_SAMPLES 134 // 12 MHz
#elif F_CPU >= 7600000
#define ADC_SAMPLES 89 // 8 MHz
#elif F_CPU >= 6000000
#define ADC_SAMPLES 82 // 7.3728 MHz
#elif F_CPU >= 3000000
#define ADC_SAMPLES 45 // 4 MHz
#elif F_CPU >= 1500000
#define ADC_SAMPLES 22 // 2 MHz
#elif F_CPU >= 300000
#define ADC_SAMPLES 11 // 1 MHz
#endif

// ADC divider for fast reading (0..7)
#define ADC_SPEED_FAST	4	// divider 16

// number of ADC samples for fast measurement with ADC_SPEED_FAST (max. 350 to avoid overflow), takes 200 us
#if F_CPU >= 14000000
#define ADC_SAMPLES_FAST 14 // 16 MHz
#elif F_CPU >= 10000000
#define ADC_SAMPLES_FAST 11 // 12 MHz
#elif F_CPU >= 7600000
#define ADC_SAMPLES_FAST 7 // 8 MHz
#elif F_CPU >= 6000000
#define ADC_SAMPLES_FAST 6 // 7.3728 MHz
#elif F_CPU >= 3000000
#define ADC_SAMPLES_FAST 4 // 4 MHz
#elif F_CPU >= 1500000
#define ADC_SAMPLES_FAST 2 // 2 MHz
#elif F_CPU >= 300000
#define ADC_SAMPLES_FAST 1 // 1 MHz
#endif

// current selected channel
u8 ADC_ChanSel;

// current selected reference voltage
//u8 ADC_RefSel;

// current selected speed
u8 ADC_SpeedSel;

// voltage of Vcc in 0.1 mV and in 0.5 mV (5.0 V), default is ADC_VCC_U
//u16 ADC_Vcc;
//u16 ADC_Vcc5;

// voltage of internal reference in 0.1 mV, measured on channel 14 (1.23V), default is ADC_BANDGAP_U
//u16 ADC_Bandgap;

// voltage of internal reference in 0.1 mV and in 0.5 mV, used for ADC sampling (2.56V), default is ADC_REF_U
//u16 ADC_Ref;
//u16 ADC_Ref5;

// read ADC sample, range 0..1023
u16 ADC_Sample();

// select new ADC channel and reference (dummy conversion and wait if voltage reference or channel has been changed)
//  channel = ADC input channel number
//  ref = reference voltage ADC_REF_VCC or ADC_REF_REF
void ADC_Sel(u8 channel);//, u8 ref);

// Read ADC samples, scale 0..1023*samples
// (should take 20 ms to eliminate industrial noise 50 Hz)
// read speed - see ADCSpeedTest function
u32 ADC_Read(u16 samples);

// Read ADC samples averages in 0.1 mV, takes 20 ms
//  channel = ADC input channel number
//  ref = reference voltage ADC_REF_VCC or ADC_REF_REF
//u16 GetU(u8 channel, u8 ref);

// set ADC conversion speed (0..7)
// - do not use speed 0 or 1 on 16 MHz, it does not work
void ADC_Speed(u8 speed);

// ADC initialize
void ADC_Init();

// recalibrate Vcc (takes 20 ms)
//void ADC_CalibVcc();
