
// ****************************************************************************
//                                 
//                                    Display
//
// ****************************************************************************

#include "include.h"

// prefix table (exp: -12, -9, -6, -3, 0, +3, +6, +9)
const u8 PrefixTab[NUM_PREFIXES] EEMEM = {'p', 'n', CHAR_MICRO, 'm', 0, 'k', 'M', 'G'};

// current cursor position (can point outside display area)
s8 DispPos = 0; // current display position
s8 DispRow = 0; // current display row

// temporary output buffer
u8 OutBuf[OUTBUF_SIZE];

// return to home position (takes 3.1 ms)
/*void DispHome()
{
	LCD_Home();
	DispPos = 0; // current display X position
	DispRow = 0; // current display Y position
}

// clear display (takes 3.1 ms)
void DispClear()
{
	LCD_Clear();
	DispPos = 0; // current display X position
	DispRow = 0; // current display Y position
}
*/
// set display output position (can draw out of display) (takes 95 us)
void DispSetPos(s8 row, s8 pos)
{
	// check if position is valid and if it has been changed
	if ((pos >= 0) && (pos < LCD_COLNUM) &&
		(row >= 0) && (row < LCD_ROWNUM) &&
		((pos != DispPos) || (row != DispRow)))
		LCD_DDRAM(row*0x40 + pos); // set new cursor position

	// save new position
	DispPos = pos;
	DispRow = row;
}

// output character to current position (takes 95 us)
void DispChar(u8 ch)
{
	// check if current position is valid
	if ((DispPos >= 0) && (DispPos < LCD_COLNUM) &&
		(DispRow >= 0) && (DispRow < LCD_ROWNUM))
		LCD_WriteData(ch); // output character

	// increase cursor position
	DispPos++;
}

// display ASCIIZ text string from ROM (takes 95 us per character)
void DispTextROM(const u8* text)
{
	u8 ch;
	for(;;)
	{
		ch = pgm_read_byte(text);
		if (ch == 0) break;
		//if (ch == SUBS_DIODE_AC) ch = CHAR_DIODE_AC;
		DispChar(ch);
		text++;
	}
}

// display text string from RAM, with length (takes 95 us per character)
void DispTextRAM(const u8* text, u8 len)
{
	for (; len > 0; len--)
	{
		DispChar(*text);
		text++;
	}
}

// display ASCIIZ text string from EEPROM (takes 95 us per character)
void DispTextEEPROM(const u8* text)
{
	u8 ch;
	for(;;)
	{
		ch = eeprom_read_byte(text);
		if (ch == 0) break;
		//if (ch == SUBS_DIODE_AC) ch = CHAR_DIODE_AC;
		DispChar(ch);
		text++;
	}
}

// display space character (takes 95 us)
void DispSpace()
{
	DispChar(' ');
}

// tabulate on position
void DispTab(u8 pos)
{
	while (DispPos < pos) DispSpace();
}

// clear one line and set cursor to start of line (takes 1.6 ms)
/*void DispClearLine(u8 row)
{
	DispSetPos(row, 0);
	u8 i;
	for (i = LCD_COLNUM; i > 0; i--) DispSpace();
	DispSetPos(row, 0);
}
*/
// clear rest of current line
void DispClearRestLine()
{
	while (DispPos < LCD_COLNUM) DispSpace();
}

// get number of digits of unsigned integer value
u8 NumberOfDigits(u32 val)
{
	u32 ref = 9;
	u8 len = 1;
	while (ref < val)
	{
		len++;
		if (ref == 999999999) break;
		ref = ref*10 + 9;
	}
	return len;
}

// decode unsigned BIN number into end of OutBuf (dig=number of digits)
/*void DecBin(u8 val, u8 dig)
{
	u8* d = &OutBuf[OUTBUF_SIZE-1];
	for (; dig > 0; dig--)
	{
		*d = (u8)((val & 1) + '0');
		d--;
		val >>= 1;
	};
}

// display BIN number (dig=number of digits) (takes dig * 95 us)
void DispBin(u8 val, u8 dig)
{
	DecBin(val, dig);
	DispTextRAM(&OutBuf[OUTBUF_SIZE - dig], dig);
}

// decode unsigned HEX number into end of OutBuf (dig=number of digits)
void DecHex(u32 val, u8 dig)
{
	u8* d = &OutBuf[OUTBUF_SIZE-1];
	u8 ch;
	for (; dig > 0; dig--)
	{
		ch = val & 0xf;
		if (ch > 9) ch += 'A' - ('9' + 1);
		ch += '0';
		*d = ch;
		d--;
		val >>= 4;
	};
}

// display hex number (dig=number of digits) (takes dig * 100 us)
void DispHex(u32 val, u8 dig)
{
	DecHex(val, dig);
	DispTextRAM(&OutBuf[OUTBUF_SIZE - dig], dig);
}
*/
// decode unsigned number into end of OutBuf, returns number of digits
u8 DecNum(u32 val)
{
	u8* d = &OutBuf[OUTBUF_SIZE-1];
	u8 len = 0;
	u8 ch;
	u32 k;
	do {
		k = val / 10;
		ch = (u8)(val - k*10);
		*d = ch + '0';
		d--;
		len++;
		val = k;
	} while (val > 0);
	return len;
}

// display unsigned integer (takes approx. 800 us)
void DispUInt(u32 val)
{
	u8 len = DecNum(val);
	DispTextRAM(&OutBuf[OUTBUF_SIZE - len], len);
}

// display signed integer (sign=always display sign character, even positive) (takes approx. 800 us)
void DispInt(s32 val, Bool sign)
{
	if (val < 0)
	{
		DispChar('-');
		val = -val;
	}
	else
		if (sign) DispChar('+');

	DispUInt(val);
}

// display unsigned value vith decimal places, prefix and unit (takes approx. 1 ms)
//   (max. 4 digits + decimal dot + prefix + unit, e.g. max. 7 chars)
// val=value
// ex=exponent related to base unit, in range -18..+8)
// unit=unit character, 0=none
void DispUVal(u32 val, s8 ex, u8 unit)
{
	u8 pref = 0; // prefix (0=none)
	u8 off = 0; // exponent offset to next 10^3 step
	u8 inx; // index ID
	u8 len; // string length
	u8* s; // text pointer

	// normalize value to 4 digits and limit lowest value
	while ((val > 9999) || (ex < -15))
	{
		val += 5; // rounding
		val /= 10; // scale value down by 10
		ex++; // increase exponent
	}

	// determine prefix and offset
	ex += 15; // shift exponent to be >=0
	inx = ex / 3; // number of 10^3 steps
	off = ex - inx*3; // offset of lower 10^3 steps
	off = 3 - off; // reverse offset 1..3

	// get prefix (inx: 0='p', 1='n', ... 7='G')
	pref = eeprom_read_byte(PrefixTab + inx);

	// decode number into buffer
	len = DecNum(val);
	s = &OutBuf[OUTBUF_SIZE - len];

	// dot position
	ex = len - off - 1;

	// prepend zero 0.
	if (ex < 0)
	{
		DispChar('0');
		DispChar('.');
		if (ex < -1) DispChar('0'); // 0.0
		if (ex < -2) DispChar('0'); // 0.00
	}

	// display value
	for (inx = 0; inx < len; inx++)
	{
		DispChar(*s++);
		if (inx == ex) DispChar('.');
	}
	
	// display prefix and unit
	if (pref != 0) DispChar(pref);
	if (unit != 0) DispChar(unit);
}

// display signed value vith decimal places, prefix and unit (takes approx. 1 ms)
//   (max. 4 digits + decimal dot + sign + prefix + unit, e.g. max. 8 chars)
// val=value
// ex=exponent related to base unit, in range -18..+8)
// unit=unit character, 0=none
// sign=always display sign character, even positive
void DispVal(s32 val, s8 ex, u8 unit, Bool sign)
{
	if (val < 0)
	{
		DispChar('-');
		val = -val;
	}
	else
		if (sign) DispChar('+');

	DispUVal(val, ex, unit);
}

// compare unsigned scaled values (returns -1 if val1 < val2, 0 if val1 == val2, +1 if val1 > val2)
/*s8 CmpValue(u32 val1, s8 scale1, u32 val2, s8 scale2)
{
	// compare number of digits
	s8 len1 = NumberOfDigits(val1) + scale1;
	s8 len2 = NumberOfDigits(val2) + scale2;

	// check if some value is zero (in such case do not normalize values)
	if ((val1 != 0) && (val2 != 0))
	{
		// val1 > val2
		if (len1 > len2) return 1;

		// val1 < val2
		if (len1 < len2) return -1;

		// length of numbers
		len1 -= scale1;
		len2 -= scale2;

		// up-scale value 2
		while (len1 > len2)
		{
			val2 *= 10;
			len2++;
		}

		// up-scale value 1
		while (len2 > len1)
		{
			val1 *= 10;
			len1++;
		}
	}

	// compare values
	if (val1 > val2) return 1;
	if (val1 < val2) return -1;
	return 0;
}
*/
// rescale value
/*u32 RescaleValue(u32 val, s8 scale, s8 newscale)
{
	// upscale
	while (newscale > scale)
	{
		val /= 10;
		scale++;
	}

	// downscale
	while (newscale < scale)
	{
		val *= 10;
		scale--;
	}

	return val;
}
*/
// display pin number 1, 2, 3
/*void DispPin(u8 pin)
{
	DispChar(pin + '1');
}
*/
