
// ****************************************************************************
//                                 
//                                    Display
//
// ****************************************************************************

// current cursor position (can point outside display area)
s8 DispPos; // current display position
s8 DispRow; // current display row

// unit prefixes
#define NUM_PREFIXES	8	// number of unit prefixes

// temporary output buffer
#define OUTBUF_SIZE 14
u8 OutBuf[OUTBUF_SIZE];

// return to home position (takes 3.1 ms)
//void DispHome();

// clear display (takes 3.1 ms)
//void DispClear();

// set display output position (can draw out of display) (takes 95 us)
void DispSetPos(s8 row, s8 pos);

// output character to current position (takes 95 us)
void DispChar(u8 ch);

// display ASCIIZ text string from ROM (takes 95 us per character)
void DispTextROM(const u8* text);

// display text string from RAM, with length (takes 95 us per character)
void DispTextRAM(const u8* text, u8 len);

// display ASCIIZ text string from EEPROM (takes 95 us per character)
void DispTextEEPROM(const u8* text);

// display space character (takes 95 us)
void DispSpace();

// tabulate on position
void DispTab(u8 pos);

// clear one line and set cursor to start of line (takes 1.6 ms)
//void DispClearLine(u8 row);

// clear rest of current line
void DispClearRestLine();

// get number of digits of unsigned integer value
u8 NumberOfDigits(u32 val);

// decode unsigned BIN number into end of OutBuf (dig=number of digits)
//void DecBin(u8 val, u8 dig);

// display BIN number (dig=number of digits) (takes dig * 95 us)
//void DispBin(u8 val, u8 dig);

// decode unsigned HEX number into end of OutBuf (dig=number of digits)
//void DecHex(u32 val, u8 dig);

// display hex number (dig=number of digits) (takes dig * 100 us)
//void DispHex(u32 val, u8 dig);

// decode unsigned number into end of OutBuf, returns number of digits
u8 DecNum(u32 val);

// display unsigned integer (takes approx. 800 us)
void DispUInt(u32 val);

// display signed integer (sign=always display sign character, even positive) (takes approx. 800 us)
void DispInt(s32 val, Bool sign);

// display unsigned value vith decimal places, prefix and unit (takes approx. 1 ms)
//   (max. 4 digits + decimal dot + prefix + unit, e.g. max. 7 chars)
// val=value
// ex=exponent related to base unit, in range -18..+8)
// unit=unit character, 0=none
void DispUVal(u32 val, s8 ex, u8 unit);

// display signed value vith decimal places, prefix and unit (takes approx. 1 ms)
//   (max. 4 digits + decimal dot + sign + prefix + unit, e.g. max. 8 chars)
// val=value
// ex=exponent related to base unit, in range -18..+8)
// unit=unit character, 0=none
// sign=always display sign character, even positive
void DispVal(s32 val, s8 ex, u8 unit, Bool sign);

// compare unsigned scaled values (returns -1 if val1 < val2, 0 if val1 == val2, +1 if val1 > val2)
//s8 CmpValue(u32 val1, s8 scale1, u32 val2, s8 scale2);

// rescale value
//u32 RescaleValue(u32 val, s8 scale, s8 newscale);

// display pin number 1, 2, 3
//void DispPin(u8 pin);
