
// ****************************************************************************
//                                 
//                              Value Filter
//
// ****************************************************************************

#include "include.h"

// delete oldest entry from the history
void FilterDel(sFilter* f)
{
	f->sum -= f->hist[f->read];
	f->read++;
	if (f->read == HISTNUM) f->read = 0;
	f->num--;
}

// get average value (should not be called before first sample)
s32 FilterGet(sFilter* f)
{
	if (f->unstable > 0) return MAX_VAL;
	if (f->sum >= 0)
		return ((f->sum + f->num/2) / f->num);
	else
		return -((-f->sum + f->num/2) / f->num);
}

// add value to value filter (returns new filtered value)
s32 FilterAdd(sFilter* f, s32 val)
{
	// limit value
	if ((val > MAX_VAL) || (val < -MAX_VAL)) val = MAX_VAL;

	// delete oldest value, if value history is full
	if (f->num == HISTNUM) FilterDel(f);

	// empty history, if value has been changed significantly
	if (f->num > 0)
	{
		s32 val2 = f->sum / f->num; // current average value
		s32 dif = val - val2; // change of the value
		if (dif < 0) dif = -dif; // absolute value of the change
		s32 val3 = val2 / 3; // 30% of current value
		if (val3 < 0) val3 = -val3; // 30% of current absolute value
		if (	((dif > 10) && // do not reset due small changes around zero
			(dif > val3)) || // value must be changed by 30% at least
			(val == MAX_VAL) || // or invalid new value
			(val2 == MAX_VAL)) // or invalid old value
		{
			f->num = 0;
			f->write = 0;
			f->read = 0;
			f->unstable += 1;
			if (f->unstable > 5) f->unstable = 5;
			f->sum = 0;
		}
	}

	// add to sum and check overflow
	if ((f->sum >= 0) && (val >= 0)) // both values are positive
	{
		f->sum += val;
		while (f->sum < val) // while positive overflow
		{
			// delete one entry
			FilterDel(f);
		}
	}
	else
	{
		if ((f->sum <= 0) && (val <= 0)) // both values are negative
		{
			f->sum += val;
			while (f->sum > val) // while negative overflow
			{
				// delete one entry
				FilterDel(f);
			}
		}
		else
			// no overflow
			f->sum += val;
	}

	// increase new value
	f->num++;
	f->hist[f->write] = val;
	f->write++;
	if (f->write == HISTNUM) f->write = 0;
	f->unstable--;
	if (f->unstable < -5) f->unstable = -5;

	// get average value
	return FilterGet(f);
}
