
// ****************************************************************************
//                                 
//                                Main function
//
// ****************************************************************************

#include "include.h"

const u8 TextTare[] EEMEM = ":TARE";

// flag - do tare
Bool DoTare = False;

// tare start time
u32 TareTime;

// ESR indicated
Bool ESR_Ind = False;

void main()
{
#ifndef DO_RPU
	// disable all pull-up resistors
	SFIOR |= BIT(PUD);
#endif

	// initialize LCD display
	LCD_Init();

	// initialize ADC
	ADC_Init();

	// initialize timer
	Timer_Init();

	// pull-up tare button
	PORTB |= B3;

	// initialize LC meter
#ifndef DO_TEST // do some test (reduce programs)
	LC_Init();
#endif // DO_TEST
	// enable global interrupt
	sei();

//#ifdef DO_CALIBREF
	// calibration by voltage reference 2.5V
//	CalibRef();
//#endif // DO_CALIBREF

#ifdef DO_ADCSPEED
	// do ADC speed test
	ADCSpeedTest();
#endif // DO_ADCSPEED

#ifdef DO_CLOCKTEST
	// do clock test (compare with real time clocks)
	ClockTest();
#endif // DO_CLOCKTEST

#ifdef DO_PRECTEST
	// do ADC speed precission test
	PrecTest();
#endif // DO_PRECTEST

//#ifdef DO_SELTEST
	// do ADC selection test
//	SelTest();
//#endif // DO_SELTEST

#ifdef DO_RPU
	// check pull-up resistor (35.5 kOhm)
	RpuTest();
#endif // DO_RPU

#ifdef DO_CHECKLEN
	// get length of checks
	CheckCheck();
#endif // DO_CHECKLEN


#ifndef DO_TEST // do some test (reduce programs)
//	s16 i;

	s32 tare;
	Bool tare2;
	Bool buz = False;

	while (1)
	{
		// recalibrate Vcc (takes 40 ms)
		//ADC_CalibVcc();

		// check tare
		if ((PINB & B3) == 0)
		{
			DoTare = True;
			TareTime = GetTime();
			if (!buz)
			{
				BuzzerTone(1200, 50);
				buz = True;
			}
		}
		tare = GetTime() - TareTime;
		tare2 = False;
		if (DoTare)
		{
			if (tare >= 1000000)
			{
				buz = False;
				tare2 = True;
			}
			if (tare >= 2000000) DoTare = False;
		}

		// 1st row
		DispSetPos(0,0);

		// check resistor
		RC_Check();

		// check capacitor
		RC_CheckCap();

		// invalid resistance
		if (ResVal == RMAX_VAL)
		{
			DispChar('-');
			DispSpace();
			DispChar(CHAR_OMEGA);
			ESR_Ind = False;
		}
		else
		{
			// R tare (max. 100 Ohm)
			if (DoTare && !ESRMode && (ResVal + RTare <= 1000L))
			{
				ESR_Ind = False;
				RTare += ResVal;
				if (tare2)
					DispVal(RTare, -1, CHAR_OMEGA, True);
				else
				{
					DispChar('R');
					DispTextEEPROM(TextTare);
				}
			}
			// display resistor
			else
			{
				DispVal(ResVal, -1, CHAR_OMEGA, False);
				if (ESRMode)
				{
					if (ResVal < 10) // 1 Ohm, excelent
					{
						DispChar(CHAR_CAP_EXC);
						if (!ESR_Ind)
						{
							BuzzerTone(300, 60);
							BuzzerTone(200, 80);
							waitms(100);
							BuzzerTone(300, 60);
							BuzzerTone(200, 80);
						}
					}
					else
					{
						if (ResVal < 60) // 6 Ohm, good
						{
							DispChar(CHAR_CAP_GOOD);
							if (!ESR_Ind)
							{
								BuzzerTone(400, 30);
								BuzzerTone(300, 40);
							}
						}
						else // >= 6 Ohm, bad
						{
							DispChar(CHAR_CAP_BAD);
							if (!ESR_Ind)
							{
								BuzzerTone(600, 40);
								BuzzerTone(800, 30);
							}
						}
					}
					ESR_Ind = True;
				}
				else
				{
					ESR_Ind = False;
					if (ResVal < 50) // < 5 Ohm
						BuzzerTone(800, 100);
				}
			}
		}

		DispTab(8);

		// invalid capacitance
		if (CapVal == CMAX_VAL)
		{
			DispChar('-');
			DispSpace();
			DispChar('F');
		}
		else
		{
			// C tare (max. 200 pF)
			if (DoTare && CapHiRes && (CapVal + CTare <= 2000L))
			{
				CTare += CapVal;
				if (tare2)
					DispVal(CTare*1000, -16, 'F', True);
				else
				{
					DispChar('C');
					DispTextEEPROM(TextTare);
				}
			}
			// display capacitor
			else
			{
				if (CapHiRes)
				{
					DispVal(CapVal*1000, -16, 'F', False); // CapVal is in 0.1 pF
					if (CapVal > 300000L) CapHiRes = False; // limit 30 nF
				}
				else
				{
					DispVal(CapVal, -11, 'F', False); // CapVal is in 10 pF
					if (CapVal < 2000L) CapHiRes = True; // limit 20 nF
				}
			}
		}
		DispClearRestLine();

		// 2nd row
		DispSetPos(1,0);

		// read L
		LC_Read();

		// invalid inductance
		if (LC_Freq < 5000L)
		{
			DispChar('-');
			DispSpace();
			DispChar('H');
		}
		else
		{
			// L tare (max. 10 uH)
			if (DoTare && (LTare + LC_L <= 1000L))
			{
				LTare += LC_L;
				if (tare2)
					DispVal(LTare, -8, 'H', True);
				else
				{
					DispChar('L');
        				DispTextEEPROM(TextTare);
				}
			}
			else
				// display inductor
				DispVal(LC_L, -8, 'H', False);
		}

		DispTab(8);

#ifdef DO_CHECKFREQ	// display LC frequency instead of U voltage
		DispUVal(LC_Freq, 0,'H');
		DispChar('z');
#else
		// check voltage (takes 40 ms)
		U_Check();

		// invalid voltage
		if (VoltVal == MAX_VAL)
		{
			DispChar('-');
			DispSpace();
			DispChar('V');
		}
		else
		{
			// U tare (max. 2V)
			if (DoTare && (UTare + VoltVal <= 2000L) && (UTare + VoltVal >= -2000L))
			{
				UTare = (UTare + VoltVal + 10000L + 500)/1000*1000 - 10000L;
				if (tare2)
					DispVal(UTare, -3, 'V', True);
				else
				{
					DispChar('U');
       					DispTextEEPROM(TextTare);
				}
			}
			else
				// display voltage
				DispVal(VoltVal, -3,'V', True);
		}
#endif
		DispClearRestLine();
	}

#endif // DO_TEST
}
