
// ****************************************************************************
//                                 
//                               Probing testpins
//
// ****************************************************************************

// test resistor indices
enum {
	R1_INX = 0,
	R2_INX,
	R3_INX,
	R4_INX,
	R5_INX,
	R6_INX,
	R7_INX,
	R8_INX,
	R8B_INX,

	RX_NUM
};

// test resistor identifiers
#define R1	B0	// 100
#define R2	B1	// 470
#define R3	B2	// 2K2
#define R4	B3	// 10K
#define R5	B4	// 47K
#define R6	B5	// 220K
#define R7	B6	// 1M0
#define R8	B7	// 4M7

#define RALL	(R1+R2+R3+R4+R5+R6+R7+R8) // all resistors identifiers

// test resistor ports and bits
#define RX_PORT	PORTD
#define RX_DDR	DDRD
#define RX_PIN	PIND

#define R1_MASK	B0
#define R2_MASK	B1
#define R3_MASK	B2
#define R4_MASK	B3
#define R5_MASK	B4
#define R6_MASK	B6
#define R7_MASK	B7

#define RX_PORT2 PORTC
#define RX_DDR2	DDRC
#define RX_PIN2	PINC

#define R8_MASK	B5

#define RALL_MASK1 (R1_MASK+R2_MASK+R3_MASK+R4_MASK+R5_MASK+R6_MASK+R7_MASK) // mask of all resistors 1
#define RALL_MASK2 R8_MASK // mask of all resistors 2

// set resistors
#define R1_OUT()	RX_DDR |= R1_MASK	// set R1 to output mode
#define R1_IN()		RX_DDR &= ~R1_MASK	// set R1 to input mode
#define R1_HIGH()	RX_PORT |= R1_MASK	// set R1 to HIGH state
#define R1_LOW()	RX_PORT &= ~R1_MASK	// set R1 to LOW state
#define R1_FLIP()	RX_PIN |= R1_MASK	// flip R1 output state

#define R2_OUT()	RX_DDR |= R2_MASK	// set R2 to output mode
#define R2_IN()		RX_DDR &= ~R2_MASK	// set R2 to input mode
#define R2_HIGH()	RX_PORT |= R2_MASK	// set R2 to HIGH state
#define R2_LOW()	RX_PORT &= ~R2_MASK	// set R2 to LOW state
#define R2_FLIP()	RX_PIN |= R2_MASK	// flip R2 output state

#define R3_OUT()	RX_DDR |= R3_MASK	// set R3 to output mode
#define R3_IN()		RX_DDR &= ~R3_MASK	// set R3 to input mode
#define R3_HIGH()	RX_PORT |= R3_MASK	// set R3 to HIGH state
#define R3_LOW()	RX_PORT &= ~R3_MASK	// set R3 to LOW state
#define R3_FLIP()	RX_PIN |= R3_MASK	// flip R3 output state

#define R4_OUT()	RX_DDR |= R4_MASK	// set R4 to output mode
#define R4_IN()		RX_DDR &= ~R4_MASK	// set R4 to input mode
#define R4_HIGH()	RX_PORT |= R4_MASK	// set R4 to HIGH state
#define R4_LOW()	RX_PORT &= ~R4_MASK	// set R4 to LOW state
#define R4_FLIP()	RX_PIN |= R4_MASK	// flip R4 output state

#define R5_OUT()	RX_DDR |= R5_MASK	// set R5 to output mode
#define R5_IN()		RX_DDR &= ~R5_MASK	// set R5 to input mode
#define R5_HIGH()	RX_PORT |= R5_MASK	// set R5 to HIGH state
#define R5_LOW()	RX_PORT &= ~R5_MASK	// set R5 to LOW state
#define R5_FLIP()	RX_PIN |= R5_MASK	// flip R5 output state

#define R6_OUT()	RX_DDR |= R6_MASK	// set R6 to output mode
#define R6_IN()		RX_DDR &= ~R6_MASK	// set R6 to input mode
#define R6_HIGH()	RX_PORT |= R6_MASK	// set R6 to HIGH state
#define R6_LOW()	RX_PORT &= ~R6_MASK	// set R6 to LOW state
#define R6_FLIP()	RX_PIN |= R6_MASK	// flip R6 output state

#define R7_OUT()	RX_DDR |= R7_MASK	// set R7 to output mode
#define R7_IN()		RX_DDR &= ~R7_MASK	// set R7 to input mode
#define R7_HIGH()	RX_PORT |= R7_MASK	// set R7 to HIGH state
#define R7_LOW()	RX_PORT &= ~R7_MASK	// set R7 to LOW state
#define R7_FLIP()	RX_PIN |= R7_MASK	// flip R7 output state

#define R8_OUT()	RX_DDR2 |= R8_MASK	// set R8 to output mode
#define R8_IN()		RX_DDR2 &= ~R8_MASK	// set R8 to input mode
#define R8_HIGH()	RX_PORT2 |= R8_MASK	// set R8 to HIGH state
#define R8_LOW()	RX_PORT2 &= ~R8_MASK	// set R8 to LOW state
#define R8_FLIP()	RX_PIN2 |= R8_MASK	// flip R8 output state

#define RALL_OUT1()	RX_DDR |= RALL_MASK1	// set R all 1 to output mode
#define RALL_IN1()	RX_DDR &= ~RALL_MASK1	// set R all 1 to input mode
#define RALL_HIGH1()	RX_PORT |= RALL_MASK1	// set R all 1 to HIGH state
#define RALL_LOW1()	RX_PORT &= ~RALL_MASK1	// set R all 1 to LOW state
#define RALL_FLIP1()	RX_PIN |= RALL_MASK1	// flip R all 1 output state

#define RALL_OUT2()	RX_DDR2 |= RALL_MASK2	// set R all 2 to output mode
#define RALL_IN2()	RX_DDR2 &= ~RALL_MASK2	// set R all 2 to input mode
#define RALL_HIGH2()	RX_PORT2 |= RALL_MASK2	// set R all 2 to HIGH state
#define RALL_LOW2()	RX_PORT2 &= ~RALL_MASK2	// set R all 2 to LOW state
#define RALL_FLIP2()	RX_PIN2 |= RALL_MASK2	// flip R all 2 output state

#define RALL_OUT()	{ RALL_OUT1(); RALL_OUT2(); }	// set R all to output mode
#define RALL_IN()	{ RALL_IN1(); RALL_IN2(); }	// set R all to input mode
#define RALL_HIGH()	{ RALL_HIGH1(); RALL_HIGH2(); }	// set R all to HIGH state
#define RALL_LOW()	{ RALL_LOW1(); RALL_LOW2(); }	// set R all to LOW state
#define RALL_FLIP()	{ RALL_FLIP1(); RALL_FLIP2(); } // flip R all output state

// time of SetRes function
// - takes 14.7 us on 16 MHz, 18.9 us on 12 MHz, 28.4 us on 8 MHz, 30.8 us on 7.3728 MHz,
//   56.8 us on 4 MHz, 113.6 us on 2 MHz, 90 us on 1 MHz
#if F_CPU >= 14000000
#define SETRES_US 15 // 16 MHz
#elif F_CPU >= 10000000
#define SETRES_US 19 // 12 MHz
#elif F_CPU >= 7600000
#define SETRES_US 28 // 8 MHz
#elif F_CPU >= 6000000
#define SETRES_US 31 // 7.3728 MHz
#elif F_CPU >= 3000000
#define SETRES_US 57 // 4 MHz
#elif F_CPU >= 1500000
#define SETRES_US 114 // 2 MHz
#elif F_CPU >= 300000
#define SETRES_US 227 // 1 MHz
#endif

// set test resistors (LOW and HIGH state)
//	0 = put resistor to high impedance
//	low = put resistor to LOW (0V)
//	high = put resistor to HIGH (+5V)
//	low + high = put resistor to pull-up (via internal resistor)
void SetRes(u8 low, u8 high);

// set all resistors to high impedance
void SetResOff();
