
// ****************************************************************************
//                                 
//                                Debug test
//
// ****************************************************************************

#include "include.h"

#ifdef DO_CALIBREF
// do voltage reference calibration using external reference TS431 2.5V, connected to GND and Rx input
// Wait a minute to stabilise values, then do set ADC_REF_U, ADC_BANDGAP_U, ADC_VCC_U in config.h
// You can adjust voltage reference value ADC_CAL_U in config.h (2.5000V default).
//
// Display values:
//  1st row:
//    - real value of ADC_REF_U (set in adc.h)
//    - real value of ADC_BANDGAP_U measured using internal reference (set in adc.h)
//    - error difference measuring external reference using internal reference
// 2nd row:
//    - real value of ADC_VCC_U (set in adc.h)
//    - real value of ADC_BANDGAP_U measured using Vcc (set in adc.h)
//    - error difference measuring external reference using Vcc
/*
const u8 TextTestRef[] EEMEM = "Connect 2.5V...";
void CalibRef()
{
	u32 i, v, v1, v2, v3, v4, v5, v6;
	i = v1 = v2 = v3 = v4 = v5 = v6 = 0;

	SetRes(0, R3); // select resistor 10K

	while (1)
	{
		DispSetPos(0,0);

		// check reference voltage
		v = GetU(ADC_MUX_IN, ADC_REF_VCC);
		if ((v < ADC_CAL_U-1500) || (v > ADC_CAL_U+1500))
		{
			i = v1 = v2 = v3 = v4 = v5 = v6 = 0;
			ADC_Vcc = ADC_VCC_U;
			ADC_Vcc5 = (ADC_VCC_U+2)/5;
			ADC_Bandgap = ADC_BANDGAP_U;
			ADC_Ref = ADC_REF_U;
			ADC_Ref5 = (ADC_REF_U+2)/5;

			DispTextEEPROM(TextTestRef);
			DispClearRestLine();
			DispSetPos(1,0);
			DispUVal(v, -4, 'V');
			DispClearRestLine();
			continue;
		}

		i++;

		// read calibration voltage 2.5V using internal reference 2.56V
		v = GetU(ADC_MUX_IN, ADC_REF_REF);
		v1 += (s32)(v - ADC_CAL_U);

		// calculate real internal reference 2.56V (display real value of ADC_REF_U)
		v = ((u32)ADC_Ref*ADC_CAL_U + v/2)/v;
		v2 += v;
		ADC_Ref = (u16)((v2 + i/2)/i);
		ADC_Ref5 = (ADC_Ref + 2)/5;
		DispUInt(ADC_Ref);
		DispSpace();

		// measure internal bandgap reference 1.23V using 2.56V reference (display real value of ADC_BANDGAP_U)
		v = GetU(ADC_MUX_BANDGAP, ADC_REF_REF);
		v3 += v;
		ADC_Bandgap = (u16)((v3 + i/2)/i);
		DispUInt(ADC_Bandgap);
		DispSpace();

		// display error difference (be aware, WinAVR does not support signed divide of s32)
		if ((s32)v1 < 0)
			DispInt(-(((u32)-v1 + i/2)/i), True);
		else
			DispInt((v1 + i/2)/i, True);
		DispClearRestLine();

		// read calibration voltage 2.5V using Vcc
		DispSetPos(1,0);
		v = GetU(ADC_MUX_IN, ADC_REF_VCC);
		v4 += (s32)(v - ADC_CAL_U);

		// calculate real Vcc (display real value of ADC_VCC_U)
		v = ((u32)ADC_Vcc*ADC_CAL_U + v/2)/v;
		v5 += v;
		ADC_Vcc = (u16)((v5 + i/2)/i);
		ADC_Vcc5 = (ADC_Vcc + 2)/5;
		DispUInt(ADC_Vcc);
		DispSpace();

		// measure internal bandgap reference 1.23V using Vcc (display real value of ADC_BANDGAP_U)
		v = GetU(ADC_MUX_BANDGAP, ADC_REF_VCC);
		v6 += v;
		ADC_Bandgap = (u16)((v6 + i/2)/i);
		DispUInt(ADC_Bandgap);
		DispSpace();

		// display error difference (be aware, WinAVR does not support signed divide of s32)
		if ((s32)v4 < 0)
			DispInt(-(((u32)-v4 + i/2)/i), True);
		else
			DispInt((v4 + i/2)/i, True);
		DispClearRestLine();
	}
}
*/
#endif // DO_CALIBREF

#ifdef DO_ADCSPEED
// do ADC speed test (displays delay of selected ADC speed,
//  time of ADC_Sample and SetRes (set SETRES_US constant)
void ADCSpeedTest()
{
	u32 acc1 = 0;	// accumulator of sample time
	u32 acc2 = 0;	// accumulator of static time
	u32 acc3 = 0;
	u32 num = 0;	// number of measures
	u32 v, i;
	u32 tim, dif;

	// 16 MHz, SetRes=14.67 us:
	//   0,1 (divider 2) ... N*3.532 us + 4.288 us, ADC_Sample = 2.821 us
	//   2 (divider 4) ... N*5.045 us + 4.530 us, ADC_Sample = 4.336 us
	//   3 (divider 8) ... N*8.538 us + 4.570 us, ADC_Sample = 8.077 us
	//   4 (divider 16) ... N*15.06 us + 5.025 us, ADC_Sample = 14.22 us
	//   5 (divider 32) ... N*28.06 us + 6.064 us, ADC_Sample = 28.05 us
	//   6 (divider 64) ... N*56.11 us + 4.121 us, ADC_Sample = 56.10 us
	//   7 (divider 128) ... N*112.0 us + 0.222 us, ADC_Sample = 112.0 us

	// 12 MHz:
	//   0,1 (divider 2) ... N*4.712 us + 5.724 us, ADC_Sample = 3.763 us
	//   2 (divider 4) ... N*6.730 us + 6.044 us, ADC_Sample = 5.784 us
	//   3 (divider 8) ... N*11.39 us + 6.107 us, ADC_Sample = 10.79 us
	//   4 (divider 16) ... N*20.08 us + 6.694 us, ADC_Sample = 18.96 us
	//   5 (divider 32) ... N*37.41 us + 8.101 us, ADC_Sample = 37.40 us
	//   6 (divider 64) ... N*74.81 us + 5.490 us, ADC_Sample = 74.80 us
	//   7 (divider 128) ... N*149.3 us + 0.295 us, ADC_Sample = 149.3 us

	// 8 MHz:
	//   0,1 (divider 2) ... N*7.066 us + 8.580 us, ADC_Sample = 5.642 us
	//   2 (divider 4) ... N*10.09 us + 9.064 us, ADC_Sample = 8.673 us
	//   3 (divider 8) ... N*17.08 us + 9.143 us, ADC_Sample = 16.18 us
	//   4 (divider 16) ... N*30.12 us + 10.04 us, ADC_Sample = 28.44 us
	//   5 (divider 32) ... N*56.12 us + 12.16 us, ADC_Sample = 56.11 us
	//   6 (divider 64) ... N*112.2 us + 8.240 us, ADC_Sample = 112.2 us
	//   7 (divider 128) ... N*224.0 us + 0.457 us, ADC_Sample = 224.0 us

	// 7.3728 MHz:
	//   0,1 (divider 2) ... N*7.670 us + 9.315 us, ADC_Sample = 6.126 us
	//   2 (divider 4) ... N*10.96 us + 9.839 us, ADC_Sample = 9.415 us
	//   3 (divider 8) ... N*18.53 us + 9.941 us, ADC_Sample = 17.56 us
	//   4 (divider 16) ... N*32.69 us + 10.90 us, ADC_Sample = 30.87 us
	//   5 (divider 32) ... N*60.89 us + 13.20 us, ADC_Sample = 60.89 us
	//   6 (divider 64) ... N*121.8 us + 8.916 us, ADC_Sample = 121.8 us
	//   7 (divider 128) ... N*243.1 us + 0.496 us, ADC_Sample = 243.1 us

	// 4 MHz:
	//   0,1 (divider 2) ... N*14.13 us + 17.16 us, ADC_Sample = 11.29 us
	//   2 (divider 4) ... N*20.19 us + 18.13 us, ADC_Sample = 17.34 us
	//   3 (divider 8) ... N*34.16 us + 18.28 us, ADC_Sample = 32.32 us
	//   4 (divider 16) ... N*60.24 us + 20.10 us, ADC_Sample = 56.89 us
	//   5 (divider 32) ... N*112.2 us + 24.26 us, ADC_Sample = 112.2 us
	//   6 (divider 64) ... N*224.4 us + 16.48 us, ADC_Sample = 224.0 us
	//   7 (divider 128) ... N*448.0 us + 0.053 us, ADC_Sample = 448.0 us

	// 2 MHz:
	//   0,1 (divider 2) ... N*28.27 us + 34.34 us, ADC_Sample = 22.57 us
	//   2 (divider 4) ... N*40.38 us + 36.26 us, ADC_Sample = 34.71 us
	//   3 (divider 8) ... N*68.33 us + 36.58 us, ADC_Sample = 64.74 us
	//   4 (divider 16) ... N*120.5 us + 40.20 us, ADC_Sample = 113.8 us
	//   5 (divider 32) ... N*224.5 us + 48.52 us, ADC_Sample = 224.4 us
	//   6 (divider 64) ... N*448.9 us + 32.83 us, ADC_Sample = 448.8 us
	//   7 (divider 128) ... N*896.1 us + 0.129 us, ADC_Sample = 896.0 us

	// 1 MHz:
	//   0,1 (divider 2) ... N*56.54 us + 68.67 us, ADC_Sample = 45.15 us
	//   2 (divider 4) ... N*80.76 us + 72.57 us, ADC_Sample = 69.41 us
	//   3 (divider 8) ... N*136.7 us + 73.14 us, ADC_Sample = 129.3 us
	//   4 (divider 16) ... N*240.9 us + 80.45 us, ADC_Sample = 227.6 us
	//   5 (divider 32) ... N*449.1 us + 97.06 us, ADC_Sample = 448.8 us
	//   6 (divider 64) ... N*897.7 us + 65.74 us, ADC_Sample = 877.7 us
	//   7 (divider 128) ... N*1792 us + 0.538 us, ADC_Sample = 1.792 us

	// set ADC speed (use value 0..7)
	ADC_Speed(DO_ADCSPEED);

	ADC_Sel(ADC_MUX_BANDGAP); //, ADC_REF_VCC);

	while (1)
	{
		// check change channel speed (should be values approx. 0, 0, 0, 260, 260, 260)
		// - to check it, do comment reading dummy sample in the ADC_Sel function
#if 0
		u32 v1, v2, v3, v4, v5, v6;
		ADC_Sel(ADC_MUX_GND); //, ADC_REF_VCC);
		v1 = ADC_Sample();
		v2 = ADC_Sample();		
		v3 = ADC_Sample();		
		ADC_Sel(ADC_MUX_BANDGAP); //, ADC_REF_VCC);
		v4 = ADC_Sample();
		v5 = ADC_Sample();		
		v6 = ADC_Sample();		
		DispSetPos(0,0);
		DispUInt(v1);
		DispSpace();
		DispUInt(v2);
		DispSpace();
		DispUInt(v3);
		DispClearRestLine();
		DispSetPos(1,0);
		DispUInt(v4);
		DispSpace();
		DispUInt(v5);
		DispSpace();
		DispUInt(v6);
		DispClearRestLine();
		continue;
#endif

		// synchronize
		v = ADC_Read(10);

		// measure time per 1 sample
		tim = GetTime();
		v = ADC_Read(1000);
		dif = GetTime() - tim - GETTIME_US;
		acc1 += dif;

		// measure time per calling
		tim += dif;
		for (i = 1000; i > 0; i--) v = ADC_Read(1);
		dif = GetTime() - tim - GETTIME_US;
		acc2 += dif;

		// measure ADC_Sample();
		tim = GetTime();
		for (i = 100; i > 0; i--)
		{	v = ADC_Sample(); v = ADC_Sample(); v = ADC_Sample(); v = ADC_Sample(); v = ADC_Sample();
			v = ADC_Sample(); v = ADC_Sample(); v = ADC_Sample(); v = ADC_Sample(); v = ADC_Sample(); }
		dif = GetTime() - tim - GETTIME_US;
		acc3 += dif;

		num++;

		// display values
		DispSetPos(0,0);
		DispUVal(acc1/num, -9, 's');
		DispChar('+');
		DispUVal((acc2 - acc1)/num, -9, 's');
		DispClearRestLine();

		DispSetPos(1,0);
		//DispTextEEPROM(TextLoop);
		//DispUInt(num);
		DispUVal(acc3/num, -9, 's');

		// measure time of SetRes (19 us on 12 MHz)
		DispSpace();
		tim = GetTime();
		for (i = 100; i > 0; i--)
		{	SetRes(0, R1); SetRes(0, R2); SetRes(0, R3); SetRes(0, R4); SetRes(R1, R4); 
			SetRes(R1, 0); SetRes(R2, 0); SetRes(R3, 0); SetRes(R4, 0); SetRes(R4, R1); }
		dif = GetTime() - tim;
		DispUVal(dif, -9, 's');
		SetResOff();

		DispClearRestLine();
	}
}
#endif // DO_ADCSPEED

#ifdef DO_CLOCKTEST
// do clock test - compare timer with seconds of real time clocks,
//   displays delay of GetTime (set GETTIME_US constant), waitus and waitms
void ClockTest()
{
	u32 tim, tim2, tim3;
	u16 tt = 0;
	u32 t1 = 0;
	u32 t2 = 0;
	u32 t3 = 0;

	u16 i;
	tim = GetTime();

	while(1)
	{
#if 0
	// display parameters of timer prescaler
		DispSetPos(0,0);
		DispUInt(SYSTIME_PRESCALER); // index of prescaler 1..5
		DispSpace();
		DispUInt(SYSTIME_DIV); // prescaler divider
		DispSpace();
		DispUVal((u32)SYSTIME_FREQ, 0, 'H');
		DispChar('z');
		DispClearRestLine();

		DispSetPos(1,0);
		DispUInt(SYSTIME_HIGH); // interrupt period HIGH (in us)
		DispSpace();
		DispUInt(SYSTIME_LOW); // interrupt period LOW (in 0.1 ns)
		DispSpace();
		DispUVal(SYSTIME_RES, -9, 's');
		DispClearRestLine();
		continue;
#endif

		DispSetPos(0,0);

		// wait for GetTime() 1 sec, display time counter
		do {
			tim2 = GetTime();
		} while ((s32)(tim2 - tim) < (s32)1000000UL);
		tim += 1000000UL;
		tt++;
		DispUInt(tt);

		DispSpace();

		// measure GetTime
		tim2 = GetTime();
		for (i = 100; i > 0; i--)
		{
			tim3 = GetTime(); tim3 = GetTime();
			tim3 = GetTime(); tim3 = GetTime();
			tim3 = GetTime(); tim3 = GetTime();
			tim3 = GetTime(); tim3 = GetTime();
			tim3 = GetTime(); tim3 = GetTime();
		}
		t1 += tim3 - tim2;
		DispUVal(t1/tt, -9, 's');

		DispClearRestLine();

		// measure waitus
		DispSetPos(1,0);
		tim2 = GetTime();
		waitus(10000);
		t2 += GetTime() - tim2 - GETTIME_US;
		DispUVal(t2/tt, -10, 's');

		DispSpace();

		// measure waitms
		tim2 = GetTime();
		waitms(10);
		t3 += GetTime() - tim2 - GETTIME_US;
		DispUVal(t3/tt, -7, 's');

		DispClearRestLine();
	}
}
#endif // DO_CLOCKTEST

#ifdef DO_PRECTEST
// do ADC speed precission test (displays value*10 + errors in 0.1 samples of divider 6, 5, 4, 3, 2, 1)
// select test resistors: 1=R1+R2, 2=R2+R3, 3=R3+R4, 4=R4+R5, 5=R5+R6,
//      6=R6+R7, 7=R7+R8, 8=R7+R6, 9=R5+R4, 10=R3+R2
void PrecTest()
{
	s32 v1, v2;

	// select resistor input and Vcc reference channel
	ADC_Sel(ADC_MUX_IN); //, ADC_REF_VCC);

	// set resistors
#if DO_PRECTEST==1
	SetRes(R1, R2); // 2030, 0, 0, 0, 10, -40, 8200
#elif DO_PRECTEST==2
	SetRes(R2, R3); // 1850, 0, 0, -10, -10, 63, 8380
#elif DO_PRECTEST==3
	SetRes(R3, R4); // 1850, 0, 0, -4, -10, 69, 8380
#elif DO_PRECTEST==4
	SetRes(R4, R5); // 1789, 0, 0, 0, 0, 49, 8440
#elif DO_PRECTEST==5
	SetRes(R5, R6); // 1798, -3, -8, -8, -8, 41, 8431
#elif DO_PRECTEST==6
	SetRes(R6, R7); // 1839, 0, 0, 0, 0, 0, 8390
#elif DO_PRECTEST==7
	SetRes(R7, R8); // 1721, 0, -1, -1, 13, 23, 8508
#elif DO_PRECTEST==8
	SetRes(R7, R6); // 8394, -4, -4, -4, -4, -84, 1835
#elif DO_PRECTEST==9
	SetRes(R5, R4); // 8439, 0, 0, 0, 0, +19, +1790
#else
	SetRes(R3, R2); // 8380, 0, 9, 9, 9, -70, 1849
#endif

#define PRECTEST_SAMPLE 5000
#define PRECTEST_DIV (PRECTEST_SAMPLE/10)

	while(1)
	{
		DispSetPos(0, 0);

		// speed 7
		ADC_Speed(7);
		waitms(2);
		v1 = ADC_Read(PRECTEST_SAMPLE);
		DispUInt(v1/PRECTEST_DIV);
		DispSpace();

		// speed 6
		ADC_Speed(6);
		v2 = ADC_Read(PRECTEST_SAMPLE);
		DispInt((v2-v1)/PRECTEST_DIV, True);
		DispSpace();

		// speed 5
		ADC_Speed(5);
		v2 = ADC_Read(PRECTEST_SAMPLE);
		DispInt((v2-v1)/PRECTEST_DIV, True);
		DispSpace();

		// speed 4
		ADC_Speed(4);
		v2 = ADC_Read(PRECTEST_SAMPLE);
		DispInt((v2-v1)/PRECTEST_DIV, True);

		DispClearRestLine();

		DispSetPos(1, 0);

		// speed 3
		ADC_Speed(3);
		v2 = ADC_Read(PRECTEST_SAMPLE);
		DispInt((v2-v1)/PRECTEST_DIV, True);
		DispSpace();

		// speed 2
		ADC_Speed(2);
		v2 = ADC_Read(PRECTEST_SAMPLE);
		DispInt((v2-v1)/PRECTEST_DIV, True);
		DispSpace();

		// speed 1
		ADC_Speed(1);
		v2 = ADC_Read(PRECTEST_SAMPLE);
		DispInt((v2-v1)/PRECTEST_DIV, True);
		DispSpace();

		DispClearRestLine();
	}
}
#endif // DO_PRECTEST

#ifdef DO_SELTEST
// do ADC selection test (displays value after reference selection, depending on delay)
//  On first row should be values approx. 492, on second row values approx. 260.
//  Values on rows should be approx. the same as the first ones on the rows.
/*void ADC_SelTest(u8 channel, u8 ref, u16 wait)
{
	ADMUX = channel | ref;
	if (wait >= 2) waitus(wait);
	ADC_Sample();
}
// 100 nF requires 1300 us, 10 nF requires 120 us, 1 nF requires 10 us
void SelTest()
{
	u32 v1, v2, v3, v4, v5, v6, v7, v8;
	v1 = v2 = v3 = v4 = v5 = v6 = v7 = v8 = 0;
	u16 n = 0;

	ADC_Speed(3); // ADC_Sample takes 8 us

	while(1)
	{
		n++;

#define SELTEST_W0 10000 // delay big enough to charge capacitor
#define SELTEST_W1 1300 // minimum for 100 nF
#define SELTEST_W2 120 // minimum for 10 nF
#define SELTEST_W3 10 // minimum for 1 nF

		ADC_SelTest(ADC_MUX_BANDGAP, ADC_REF_REF, SELTEST_W0);
		v1 += ADC_Sample();
		ADC_SelTest(ADC_MUX_BANDGAP, ADC_REF_VCC, SELTEST_W0);
		v2 += ADC_Sample();

		ADC_SelTest(ADC_MUX_BANDGAP, ADC_REF_REF, SELTEST_W1);
		v3 += ADC_Sample();
		ADC_SelTest(ADC_MUX_BANDGAP, ADC_REF_VCC, SELTEST_W1);
		v4 += ADC_Sample();

		ADC_SelTest(ADC_MUX_BANDGAP, ADC_REF_REF, SELTEST_W2);
		v5 += ADC_Sample();
		ADC_SelTest(ADC_MUX_BANDGAP, ADC_REF_VCC, SELTEST_W2);
		v6 += ADC_Sample();

		ADC_SelTest(ADC_MUX_BANDGAP, ADC_REF_REF, SELTEST_W3);
		v7 += ADC_Sample();
		ADC_SelTest(ADC_MUX_BANDGAP, ADC_REF_VCC, SELTEST_W3);
		v8 += ADC_Sample();

		DispSetPos(0, 0);
		DispUInt(v1/n);
		DispSpace();
		DispUInt(v3/n);
		DispSpace();
		DispUInt(v5/n);
		DispSpace();
		DispUInt(v7/n);
		DispClearRestLine();

		DispSetPos(1, 0);
		DispUInt(v2/n);
		DispSpace();
		DispUInt(v4/n);
		DispSpace();
		DispUInt(v6/n);
		DispSpace();
		DispUInt(v8/n);
		DispClearRestLine();
	}
}
*/
#endif // DO_SELTEST

#ifdef DO_RPU
// check pull-up resistor (measured value is 38 kOhm, it should be 30K..60K)
void RpuTest()
{
	ADC_Sel(ADC_MUX_IN); //, ADC_REF_VCC);
	ADC_Speed(ADC_SPEED);

	for (;;)
	{
		// I = Vcc/(R0+R3+R1+Rpu); I*Rpu + I*(R0+R3+R1) = Vcc; Rpu=Vcc/I - (R0+R3+R1)
		// v = I*(R0+R3); I = v/(R0+R3)
		// Rpu = Vcc/v*(R0+R3) - (R0+R3+R1)
		DispSetPos(0, 0);
		SetRes(R1+R3, R1);
		u32 v1 = ADC_Read(1000);
		v1 = ((u32)(R3_VAL*10+R0_VAL)*1023UL*100+v1/2)/v1 - R3_VAL - R0_VAL/10 - R1_VAL/10;
		DispUVal(v1, 0, CHAR_OMEGA);
		DispSpace();

		// I = Vcc/(R0+R4+R1+Rpu); I*Rpu + I*(R0+R4+R1) = Vcc; Rpu=Vcc/I - (R0+R4+R1)
		// v = I*(R0+R4); I = v/(R0+R4)
		// Rpu = Vcc/v*(R0+R4) - (R0+R4+R1)
		SetRes(R1+R4, R1);
		u32 v2 = ADC_Read(1000);
		v2 = ((u32)(R4_VAL*100+R0_VAL*10)*1023UL+v2/2)/v2*10 - R4_VAL - R0_VAL/10 - R1_VAL/10;
		DispUVal(v2, 0, CHAR_OMEGA);
		DispClearRestLine();

		// I = Vcc/(R0+R5+R1+Rpu); I*Rpu + I*(R0+R5+R1) = Vcc; Rpu=Vcc/I - (R0+R5+R1)
		// v = I*(R0+R5); I = v/(R0+R5)
		// Rpu = Vcc/v*(R0+R5) - (R0+R5+R1)
		DispSetPos(1, 0);
		SetRes(R1+R5, R1);
		u32 v3 = ADC_Read(1000);
		v3 = ((u32)(R5_VAL*10+R0_VAL*10)*1023UL+v3/2)/v3*100 - R5_VAL - R0_VAL/10 - R1_VAL/10;
		DispUVal(v3, 0, CHAR_OMEGA);
		DispSpace();

		// I = Vcc/(R0+R6+R1+Rpu); I*Rpu + I*(R0+R6+R1) = Vcc; Rpu=Vcc/I - (R0+R6+R1)
		// v = I*(R0+R6); I = v/(R0+R6)
		// Rpu = Vcc/v*(R0+R6) - (R0+R6+R1)
		SetRes(R1+R6, R1);
		u32 v4 = ADC_Read(1000);
		v4 = ((u32)(R6_VAL*10+R0_VAL)*1023UL+v4/2)/v4*100 - R6_VAL - R0_VAL/10 - R1_VAL/10;
		DispUVal(v4, 0, CHAR_OMEGA);
		DispClearRestLine();

		waitms(3000);

		DispSetPos(0, 0);
		DispChar('R');
		DispChar('p');
		DispChar('u');
		DispChar('=');
		DispUVal((v1+v2+v3+v4+2)/4, 0, CHAR_OMEGA);
		DispClearRestLine();
		DispSetPos(1, 0);
		DispClearRestLine();
		waitms(3000);
	}
}
#endif // DO_RPU

#ifdef DO_CHECKLEN
// get length of checks in ms: RC_Check, RC_DetCap, RC_CheckRes, RC_CheckESR
void CheckCheck()
{
	u32 tim;
	u32 t1, t2, t3, t4, t5, t6, n;
	t1 = t2 = t3 = t4 = t5 = t6 = n = 0;

	ADC_Sel(ADC_MUX_IN); //, ADC_REF_VCC);

	for (;;)
	{
		n++;

		DispSetPos(0,0);

		// check resistor (takes 46 ms) and capacitor
		tim = GetTime();
		RC_CheckRes();
		t1 += GetTime() - tim;
		DispUInt((t1+500*n)/(1000*n));

		// detect capacitor (takes 5 ms)
		DispSpace();
		tim = GetTime();
		RC_DetCap();
		t2 += GetTime() - tim;
		DispUInt((t2+500*n)/(1000*n));

		// check simple resistor (takes 42 ms)
		DispSpace();
		tim = GetTime();
		RC_CheckRes();
		t3 += GetTime() - tim;
		DispUInt((t3+500*n)/(1000*n));

		// check ESR (takes 20 ms)
		DispSpace();
		tim = GetTime();
		RC_CheckESR();
		t4 += GetTime() - tim;
		DispUInt((t4+500*n)/(1000*n));

	}
}
#endif // DO_CHECKLEN
