
// ****************************************************************************
//                                 
//                          System Clock (using Timer0)
//
// ****************************************************************************

#include "include.h"

// system time (number of us from restart, with period 71 minutes)
volatile u32 SysTime = 0;

// low part of system time in 0.1 ns
#ifndef NO_SYSTIME_LOW
volatile u16 SysTimeLow = 0;
#endif

// last time of update LC
volatile u32 LCUpdate = 0;

// Timer0 interrupt routine
ISR(TIMER0_OVF_vect)
{
	// increment system time
	u32 st = SysTime;
	st += SYSTIME_HIGH;
#ifndef NO_SYSTIME_LOW
	u16 stL = SysTimeLow;
	stL += SYSTIME_LOW;
	if (stL >= 10000)
	{
		stL -= 10000;
		st++;
	}
	SysTimeLow = stL;
#endif
	SysTime = st;

	// LC measure update, called every 250 ms
	u32 lcu = LCUpdate;
	lcu = st - lcu;
	if (lcu >= 250000)
	{
		LCUpdate = st;
		LC_Update(lcu);
	}
}

// initialize timer
void Timer_Init()
{
	// set prescaler
	TCCR0 = SYSTIME_PRESCALER;

	// reset counter
	TCNT0 = 0;

	// enable overflow interrupt
	TIMSK |= BIT(TOIE0);
}

// get system time in number of us from restart, with period 71 minutes
// - Temporary disables and enables interrupts
u32 GetTime()
{
	u32 tim;
	u8 tcnt;
	u8 tifr;

	do {
		cli(); // disable interrupts
		tcnt = TCNT0; // get counter
		tim = SysTime; // get system time
		tifr = TIFR; // get interrupt flag register
		sei(); // enable interrupts
	} while ((tifr & BIT(TOV0)) != 0); // wait until interrupt has been satisfied

	// recalculate counter to time
	return tim + ((SYSTIME_HIGH*tcnt)>>8);
}
